#! /usr/bin/python

import sys
import re
import time
import os

def load_exp_file( exp_file ):
	file = open( exp_file, "r" )
	dict = {}
	for line in file.readlines():
		line = line.strip()

		# skip comments and emtpy lines
		if line == "":
			continue

		reg = re.compile( "^#" )
		if  reg.match( line ):
			continue

		a = line.split( None, 1 )
		if len(a) < 2:
			print "malformed line ",line
			continue
		type = a[0] # include|exclude
		value = a[1] 
		
		if type == "include" or type == "exclude":
			regex = value
			# compile regex, and complain if the compile fails (alerts user to bad regexes in the file)
			try:
				regex = re.compile(regex)
			except:
				print "malformed regex: ",line
				continue
		
			# append to the array of regexes for this type, creating that array if it is the first regex for that type
			if dict.has_key(type):
				list = dict[type]
				list.append(regex)
			else:
				a = [regex]
				dict[type] = a
		else:
			dict[type] = value
	
	file.close()
	# setup dummy arrays in case config file actually contains
	# no include lines or no exclude lines
	if dict.has_key("include"):
		pass
	else:
		dict["include"] = []
	if dict.has_key("exclude"):
		pass
	else:
		dict["exclude"] = []

	# debug is another key that has to be in there
	if dict.has_key("debug"):
		pass
	else:
		dict["debug"] = "false"

	return dict

def usage():
	print "usage: ",sys.argv[0],"config_file"
	print "gets the name of a log file, plus list of include/exclude regular expressions from config_file"
	print "and emails appropriate lines from the log file."
	sys.exit()


# argument handling
try:
	conf_file = sys.argv[1]
except IndexError:
	usage()


# load up our config file with its include and exclude regexes from the passed file
config = load_exp_file(conf_file)
if config.has_key("log_file"):
	pass
else:
	print "no log_file specified in config file! quiting."
	sys.exit()

if config.has_key("pid_file"):
	pass
else:
	print "no pid_file specified in config file! quiting."
	sys.exit()

if config.has_key("email_recpt"):
	pass
else:
	print "no email_recpt specified in config file! quiting."
	sys.exit()

if config.has_key("email_delay"):
	pass
else:
	print "no email_delay specified in config file! quiting."
	sys.exit()

try:
	file = open(config["pid_file"], "r")
	print "pid file", config["pid_file"], "exists, refusing to run"
	file.close()
	sys.exit(2)
except IOError:
	file = open(config["pid_file"], "w")
	pid = str(os.getpid()) + "\n"
	file.write(pid)
	file.close()




# mark our starting time
start = time.time()
# init our array of lines to be sent
message = []
# our count of messages sent
messages_sent = 0

# record the inode of our file
fstat = os.stat( config["log_file"] )
inode = fstat[1]

# open our log file and begin cycling throug it
file = open(config["log_file"], "r")
while 1:
	line = file.readline()

	# check if it is time to send a message
	if time.time() >= start + float(config["email_delay"]):
		if config["debug"] == "true":
			print "TIME TO SEND OUT MESSAGE"
		if len(message) > 1:
			if config["debug"] == "true":
				print "MESSAGE BUFFER CONTAINS DATA"
			messages_sent = messages_sent + 1
			command = "mail -s \"" + config["log_file"] + ": message(s) of interest " + str(messages_sent) + "\" " + config["email_recpt"]
			if config["debug"] == "true":
				print "MAILING OUT MESSAGE"
			mail = os.popen( command, "w" )
			for item in message:
				mail.write(item + "\n")
			mail.close()
			if config["debug"] == "true":
				print "MESSAGE SENT"
		else:
			if config["debug"] == "true":
				print "MESSAGE BUFFER EMPTY"
			

		# reset some variables
		start = time.time()
		message = []
		# reload the config file while we are at it
		# notice this will only pick up changes to the regex, it's too late to notice other changes
		if config["debug"] == "true":
			print "RELOADING CONFIG FILE"
		config = load_exp_file(conf_file)

		
		# now check to see if our file has been moved (aka log rotation)
		# by comparing the inode of our file descriptor
		# to the inode on the actual file
		if config["debug"] == "true":
			print "CHECKING IF FILE HAS BEEN MOVED"
		try:
			nstat = os.stat( config["log_file"] )
			if config["debug"] == "true":
				print "OLD INDODE IS", inode, "NEW INODE IS", nstat[1]
			if inode != nstat[1]:
				if config["debug"] == "true":
					print "INODE HAS CHANGED, REFRESHING FILE HANDLE"
				file.close()
				file = open(config["log_file"], "r")
				# stash new inode
				inode = nstat[1]
				# reset the message counter
				messages_sent = 0
		except OSError:	
			if config["debug"] == "true":
				print "UNABLE TO STAT FILE"
			# this happens if the file no longer exists
			# in this case we just keep on chugging
			# with the current file descriptor
			# in the hope that the file has been moved
			# and a new inode for the file will be created by the next pass through
			pass
		

	# check if we are at the end of the file
	if line == "":
		if config["debug"] == "true":
			print "REACHED THE END OF THE FILE, PAUSING"
		# if so, sleep a bit and then look for new data
		time.sleep(10)
		if config["debug"] == "true":
			print "RESUMING"
		continue

	# strip out newlines
	line = line.strip()

	if config["debug"] == "true":
		print "READ LINE:", line

	# check to see if line matches any of our 'always include' regexes
	match = "no"
	for regex in config["include"]:
		if regex.search(line):
			message.append(line)
			match = "yes"
			break
	if match == "yes":
		continue


	# check to see if line matches any of our 'to be excluded' regexes
	match = "no"
	for regex in config["exclude"]:
		if regex.search(line):
			match = "yes"
			break
	if match == "yes":
		continue

	if config["debug"] == "true":
		print "LINE ADDED TO MESSAGE BUFFER"
	message.append(line)

file.close()
		
	



