#! /bin/sh

USER=root
PKG_DIR="/usr/local"
BIN="$PKG_DIR/bin/log_sniffer.py"
ETC_DIR="$PKG_DIR/etc/log_sniffer"


FILES=$ETC_DIR/*
test -n "$2" && FILE="$2"

case "$1" in
  start)
	for config in $FILES
	do
		test -f $config || continue
		LOG_FILE=`cat $config | grep -v "^#" | grep 'log_file' | awk '{ print $2; }'`
		echo "Starting log sniffer process for file $LOG_FILE"
		su $USER $BIN $config &
	done
	;;
  stop)
	for config in $FILES
	do
		test -f $config || continue
		LOG_FILE=`cat $config | grep -v "^#" | grep 'log_file' | awk '{ print $2; }' `
		echo "Stopping log sniffer process for file $LOG_FILE"
		PID_FILE=`cat $config | grep -v "^#" | grep 'pid_file' | awk '{ print $2; }'`
		echo "pid file is $PID_FILE"
		if test -f $PID_FILE
		then
			PID=`cat "$PID_FILE"`
			echo Sending kill to process $PID
			kill -9 $PID
			rm "$PID_FILE"
		fi
	done
	;;
  *)
	echo "Usage: $0 {start|stop} [config_file]" 
	echo "starts/stops log_sniffer script"
	echo "by default, starts/stops a copy of the script for every config file found in $ETC_DIR"
	echo "if passed a config_file, starts/stops log sniffer script for just that config file"
	echo "log_sniffer scripts are run as user $USER"
	exit 1
	;;
esac

exit 0
